@@ -21,14 +21,20 @@
 #include <math.h>
 #include <stdlib.h>
 
+#include <new>
+
 #include <QApplication>
+#include <QFile>
+#include <QFileInfo>
 #include <QFrame>
 #include <QGridLayout>
 #include <QHBoxLayout>
 #include <QMimeType>
 #include <QPoint>
+#include <QPointer>
 #include <QResizeEvent>
 #include <QScrollBar>
+#include <QTextStream>
 #include <QWheelEvent>
 #include <QtGlobal>
 
@@ -41,6 +47,7 @@
 #include "libkwave/FileInfo.h"
 #include "libkwave/Label.h"
 #include "libkwave/LabelList.h"
+#include "libkwave/Logger.h"
 #include "libkwave/MessageBox.h"
 #include "libkwave/Parser.h"
 #include "libkwave/SignalManager.h"
@@ -48,6 +55,7 @@
 #include "libkwave/Utils.h"
 #include "libkwave/undo/UndoTransactionGuard.h"
 
+#include "libgui/FileDialog.h"
 #include "libgui/LabelPropertiesWidget.h"
 #include "libgui/OverViewWidget.h"
 #include "libgui/SignalWidget.h"
@@ -73,6 +81,19 @@
  */
 #define DEFAULT_DISPLAY_TIME (5 * 60.0)
 
+/**
+ * File extension used for label lists
+ */
+#define LABEL_LIST_EXT _("*.label")
+
+/**
+ * Filter for file dialogs for loading/saving labels
+ */
+#define LABEL_LIST_FILTER \
+    LABEL_LIST_EXT + _("|") + \
+    i18n("Kwave label list") + \
+    _(" (") + LABEL_LIST_EXT + _(")")
+
 //***************************************************************************
 Kwave::MainWidget::MainWidget(QWidget *parent, Kwave::FileContext &context,
                               const QSize &preferred_size)
@@ -167,20 +188,16 @@ Kwave::MainWidget::MainWidget(QWidget *parent, Kwave::FileContext &context,
 
     Kwave::PlaybackController &playback = signal_manager->playbackController();
     connect(&playback, SIGNAL(sigPlaybackPos(sample_index_t)),
-	m_overview, SLOT(playbackPositionChanged(sample_index_t)));
+	m_overview, SLOT(showCursor(sample_index_t)));
     connect(&playback, SIGNAL(sigPlaybackStopped()),
-	m_overview, SLOT(playbackStopped()));
+	m_overview, SLOT(showCursor()));
 
     // -- connect all signals from/to the signal widget --
 
     connect(&m_signal_widget, SIGNAL(sigCommand(QString)),
 	    this,             SIGNAL(sigCommand(QString)));
-    connect(&m_signal_widget,
-	    SIGNAL(sigMouseChanged(Kwave::MouseMark::Mode,
-	                           sample_index_t, sample_index_t)),
-            this,
-	    SIGNAL(sigMouseChanged(Kwave::MouseMark::Mode,
-	                           sample_index_t, sample_index_t)));
+    connect(&m_signal_widget, SIGNAL(sigCursorChanged(sample_index_t)),
+	    m_overview,       SLOT(showCursor(sample_index_t)));
 
     // -- connect all signals from/to the signal manager --
 
@@ -462,7 +479,7 @@ int Kwave::MainWidget::executeCommand(const QString &command)
 	signal_manager->selectRange(m_offset, 0);
 
     // label handling
-    CASE_COMMAND("add_label")
+    CASE_COMMAND("label:add")
 	sample_index_t pos = parser.toSampleIndex();
 	if (!parser.isDone()) {
 	    // 2 parameters: position + description
@@ -472,31 +489,26 @@ int Kwave::MainWidget::executeCommand(const QString &command)
 	    // 1 parameter only: open dialog for editing the description
 	    addLabel(pos, QString());
 	}
-    CASE_COMMAND("edit_label")
+    CASE_COMMAND("label:edit")
 	int index = parser.toInt();
 	Kwave::LabelList labels(signal_manager->metaData());
 	if ((index >= labels.count()) || (index < 0))
 	    return -EINVAL;
 	Kwave::Label label = labels.at(index);
 	labelProperties(label);
+    CASE_COMMAND("label:load")
+	QString filename = parser.nextParam();
+	return loadLabels(filename);
+   CASE_COMMAND("label:save")
+	QString filename = parser.nextParam();
+	return saveLabels(filename);
 
-//    CASE_COMMAND("chooselabel")
-//	Kwave::Parser parser(command);
-//	markertype = globals.markertypes.at(parser.toInt());
-//    CASE_COMMAND("amptolabel")
+//    CASE_COMMAND("label:by_intensity")
 //	markSignal(command);
-//    CASE_COMMAND("pitch")
-//	markPeriods(command);
-//    CASE_COMMAND("labeltopitch")
+//    CASE_COMMAND("label:to_pitch")
 //      convertMarkstoPitch(command);
-//    CASE_COMMAND("loadlabel")  -> plugin
-//	loadLabel();
-//    CASE_COMMAND("savelabel")  -> plugin
-//	saveLabel(command);
-//    CASE_COMMAND("markperiod")
+//    CASE_COMMAND("label:by_period")
 //	markPeriods(command);
-//    CASE_COMMAND("saveperiods")
-//	savePeriods();
 
     } else {
 	return (signal_manager) ?
@@ -529,7 +541,7 @@ void Kwave::MainWidget::refreshHorizontalScrollBar()
 	// calculate the scrollbar ranges in scrollbar's units
 	//
 	// NOTE: we must take care of possible numeric overflows
-	//       as the scrollbar works internally with "int" and
+	//       as the scrollbar internally works with "int" and
 	//       the offsets we use for the samples might be bigger!
 	//
 	// [-------------------------------------------##############]
@@ -880,12 +892,17 @@ void Kwave::MainWidget::addLabel(sample_index_t pos, const QString &description)
     Q_ASSERT(signal_manager);
     if (!signal_manager) return;
 
+    // remember the last "modified" state, in case that we abort
+    bool was_modified = signal_manager->isModified();
+
+    // create an own undo transaction
     Kwave::UndoTransactionGuard undo(*signal_manager, i18n("Add Label"));
 
     // add a new label, with undo
     Kwave::Label label = signal_manager->addLabel(pos, description);
     if (label.isNull()) {
 	signal_manager->abortUndoTransaction();
+	if (!was_modified) signal_manager->setModified(false);
 	return;
     }
 
@@ -895,57 +912,117 @@ void Kwave::MainWidget::addLabel(sample_index_t pos, const QString &description)
 	int index = signal_manager->labelIndex(label);
 	signal_manager->deleteLabel(index, false);
 	signal_manager->abortUndoTransaction();
+	if (!was_modified) signal_manager->setModified(false);
     }
 }
 
-// ////****************************************************************************
-// //void MainWidget::loadLabel()
-// //{
-// //    labels().clear();    //remove old Labels...
-// //    appendLabel ();
-// //}
+//****************************************************************************
+int Kwave::MainWidget::loadLabels(const QString &filename)
+{
+    Kwave::SignalManager *signal_manager = m_context.signalManager();
+    Q_ASSERT(signal_manager);
+    if (!signal_manager) return -1;
 
-// ////****************************************************************************
-// //void MainWidget::saveLabel (const char *typestring)
-// //{
-// //    QString name = QFileDialog::getSaveFileName(this, QString(), 0, "*.label");
-// //    if (!name.isNull()) {
-// //	FILE *out;
-// //	out = fopen (name.local8Bit(), "w");
-// //
-// //	Kwave::Parser parser (typestring);
-// //	Kwave::Label *tmp;
-// //	LabelType *act;
-// //
-// //	const char *actstring = parser.getFirstParam();
-// //
-// //	while (actstring) {
-// //	    printf ("selecting %s\n", actstring);
-// //	    for (act = globals.markertypes.first(); act; act = globals.markertypes.next())
-// //		if (strcmp(act->name, actstring) == 0) {
-// //		    printf ("selected\n");
-// //		    act->selected = true;
-// //		    break;
-// //		}
-// //	    actstring = parser.getNextParam();
-// //	}
-// //
-// //	for (act = globals.markertypes.first(); act; act = globals.markertypes.next())
-// //	    //write out all selected label types
-// //	    if (act->selected)
-// //		fprintf (out, "%s\n", act->getCommand());
-// //
-// //	//ended writing of types, so go on with the labels...
-// //
-// //	for (tmp = labels().first(); tmp; tmp = labels().next())  //write out labels
-// //	{
-// //	    fprintf (out, tmp->getCommand());
-// //	    fprintf (out, "\n");
-// //	}
-// //
-// //	fclose (out);
-// //    }
-// //}
+    QUrl url;
+    if (!filename.length()) {
+	QString name(filename);
+
+	QPointer<Kwave::FileDialog> dlg = new (std::nothrow)Kwave::FileDialog(
+	_("kfiledialog:///kwave_label_dir"),
+	Kwave::FileDialog::OpenFile, LABEL_LIST_FILTER, this,
+	QUrl(), LABEL_LIST_EXT);
+	if (!dlg) return -1;
+	dlg->setWindowTitle(i18n("Load Labels"));
+	if (dlg->exec() != QDialog::Accepted) {
+	    delete dlg;
+	    return 0;
+	} else {
+	    url = dlg->selectedUrl();
+	    delete dlg;
+	}
+    } else {
+	url = QUrl::fromUserInput(filename);
+    }
+
+    // create an own undo transaction
+    Kwave::UndoTransactionGuard undo(*signal_manager, i18n("Load Labels"));
+
+    return m_context.loadBatch(url);
+}
+
+//****************************************************************************
+int Kwave::MainWidget::saveLabels(const QString &filename)
+{
+    Kwave::SignalManager *signal_manager = m_context.signalManager();
+    Q_ASSERT(signal_manager);
+    if (!signal_manager) return false;
+
+    QUrl url;
+    url = QUrl(m_context.signalName());
+
+    if (!filename.length()) {
+	QString name(filename);
+
+	QPointer<Kwave::FileDialog> dlg = new(std::nothrow)Kwave::FileDialog(
+	    _("kfiledialog:///kwave_label_dir"),
+	    Kwave::FileDialog::SaveFile, LABEL_LIST_FILTER,
+	    this, url, LABEL_LIST_EXT);
+	if (!dlg) return 0;
+	dlg->setWindowTitle(i18n("Save Labels"));
+	if (dlg->exec() != QDialog::Accepted) {
+	    delete dlg;
+	    return -1;
+	}
+	url = dlg->selectedUrl();
+	if (url.isEmpty()) {
+	    delete dlg;
+	    return 0;
+	}
+	delete dlg;
+
+	// add an extension if necessary
+	name = url.path();
+	if (!QFileInfo(name).suffix().length()) {
+	    url.setPath(name + _(".label"));
+	    name = url.path();
+	}
+
+	// check if the file exists and ask before overwriting it
+	// if it is not the old filename
+	if (QFileInfo(name).exists())
+	{
+	    if (Kwave::MessageBox::warningYesNo(this,
+		i18n("The file '%1' already exists.\n"
+		"Do you really want to overwrite it?", name)) !=
+		KMessageBox::Yes)
+	    {
+		return -1;
+	    }
+	}
+    } else {
+	url = QUrl::fromUserInput(filename);
+    }
+
+    // now we have a file name -> save all labels...
+    Kwave::Logger::log(this, Kwave::Logger::Info,
+	_("saving labels to '") + url.toDisplayString() + _("'"));
+
+    QFile file(url.path());
+    file.open(QIODevice::WriteOnly);
+    QTextStream out(&file);
+
+    Kwave::LabelList labels(signal_manager->metaData());
+    foreach (const Kwave::Label &label, labels) {
+	sample_index_t pos = label.pos();
+	const QString name = Kwave::Parser::escape(label.name());
+	out << _("label:add(") << pos;
+	if (name.length()) out << _(", ") << name;
+	out << _(")") << endl;
+    }
+
+    file.close();
+    return 0;
+}
 
 //***************************************************************************
 bool Kwave::MainWidget::labelProperties(Kwave::Label &label)
@@ -1035,7 +1112,7 @@ bool Kwave::MainWidget::labelProperties(Kwave::Label &label)
 	}
 
 	// modify the label through the signal manager
-	if (!signal_manager->modifyLabel(index, new_pos, new_name)) {
+	if (!signal_manager->modifyLabel(index, new_pos, new_name, true)) {
 	    // position is already occupied
 	    signal_manager->abortUndoTransaction();
 	    return false;
@@ -1054,63 +1131,6 @@ bool Kwave::MainWidget::labelProperties(Kwave::Label &label)
     return accepted;
 }
 
-// ////****************************************************************************
-// //void MainWidget::savePeriods ()
-// //{
-// //    if (signalmanage) {
-// //	Dialog *dialog =
-// //	    DynamicLoader::getDialog ("marksave", new DialogOperation(&globals, signalmanage->getRate(), 0, 0));
-// //
-// //	if ((dialog) && (dialog->exec())) {
-// //	    selectMarkers (dialog->getCommand());
-// //
-// //	    LabelType *act;
-// //	    Kwave::Label *tmp;
-// //	    int last = 0;
-// //	    int rate = signalmanage->getRate ();
-// //
-// //	    QString name = QFileDialog::getSaveFileName(this, QString(), 0, "*.dat");
-// //	    if (!name.isNull()) {
-// //		QFile out(name.local8Bit());
-// //		char buf[160];
-// //		float freq = 0, time, lastfreq = 0;
-// //		out.open (QIODevice::WriteOnly);
-// //		int first = true;
-// //
-// //		for (act = globals.markertypes.first(); act; act = globals.markertypes.next())
-// //		    //write only selected label type
-// //		    if (act->selected)
-// //			//traverse list of all labels
-// //			for (tmp = labels->first(); tmp; tmp = labels->next()) {
-// //			    if (tmp->getType() == act) {
-// //				freq = tmp->pos - last;
-// //				time = last * 1000 / rate;
-// //
-// //				if ((!first) && (freq != lastfreq)) {
-// //				    lastfreq = freq;
-// //				    freq = 1 / (freq / rate);
-// //				    snprintf (buf, sizeof(buf), "%f %f\n",
-// //					time, freq);
-// //				    out.writeBlock (&buf[0], strlen(buf));
-// //				} else lastfreq = freq;
-// //				first = false;
-// //				last = ms2samples(tmp->pos);
-// //			    }
-// //			}
-// //
-// //		if (!first) //make sure last tone gets its length
-// //		{
-// //		    time = last * 1000 / rate;
-// //		    snprintf (buf, sizeof(buf), "%f %f\n", time, freq);
-// //		    out.writeBlock (&buf[0], strlen(buf));
-// //		}
-// //
-// //		out.close ();
-// //	    }
-// //	}
-// //    }
-// //}
-
 // ////****************************************************************************
 // //void MainWidget::markSignal (const char *str)
 // //{
