@@ -108,7 +108,7 @@ QStringList *Kwave::SaveBlocksPlugin::setup(QStringList &previous_params)
 	return 0;
     }
 
-    QStringList *list = new QStringList();
+    QStringList *list = new(std::nothrow) QStringList();
     Q_ASSERT(list);
     if (list) {
 	// user has pressed "OK"
@@ -117,6 +117,7 @@ QStringList *Kwave::SaveBlocksPlugin::setup(QStringList &previous_params)
 	QUrl url = dialog->selectedUrl();
 	if (url.isEmpty()) {
 	    delete dialog;
+	    delete list;
 	    return 0;
 	}
 	QString name = url.path();
@@ -148,10 +149,6 @@ QStringList *Kwave::SaveBlocksPlugin::setup(QStringList &previous_params)
 	    QString::number(mode) + _(",") +
 	    QString::number(selection_only) + _(")")
 	);
-    } else {
-	// user pressed "Cancel"
-	delete list;
-	list = 0;
     }
 
     if (dialog) delete dialog;
@@ -188,7 +185,7 @@ int Kwave::SaveBlocksPlugin::start(QStringList &params)
 {
     qDebug("SaveBlocksPlugin::start()");
 
-    // interprete the parameters
+    // interpret the parameters
     int result = interpreteParameters(params);
     if (result) return result;
 
@@ -205,14 +202,11 @@ int Kwave::SaveBlocksPlugin::start(QStringList &params)
     selection(0, &selection_left, &selection_right, false);
 
     bool selected_something = (selection_left != selection_right);
-    bool selected_all = ((selection_left == 0) &&
-                         (selection_right+1 >= signalLength()));
+    bool selected_all = ( (selection_left == 0) &&
+                          ((selection_right + 1) >= signalLength()) );
     bool enable_selection_only = selected_something && !selected_all;
     bool selection_only = enable_selection_only && m_selection_only;
-
-    if (selection_only) {
-	selection(0, &selection_left, &selection_right, true);
-    } else {
+    if (!selection_only) {
 	selection_left  = 0;
 	selection_right = signalLength() - 1;
     }
@@ -267,8 +261,7 @@ int Kwave::SaveBlocksPlugin::start(QStringList &params)
 
 	// check for missing subdirectory
 	if (!file_info.dir().exists()) {
-	    QFileInfo inf(display_name);
-	    QString missing_dir = inf.path();
+	    QString missing_dir = file_info.absolutePath();
 	    if (!missing_dirs.contains(missing_dir))
 		missing_dirs += missing_dir;
 	}
@@ -500,25 +493,6 @@ void Kwave::SaveBlocksPlugin::scanBlocksToSave(const QString &base,
     }
 }
 
-//***************************************************************************
-QString Kwave::SaveBlocksPlugin::escapeForFileName(const QString &text)
-{
-    QString result = text;
-
-    // convert all kinds of (multiple) whitespaces, including tabs
-    // and newlines into single spaces
-    QRegExp rx(_("[\\s\\\t\\\r\\\n]+"));
-    result.replace(rx, QChar(0x0020));
-
-    // NOTE: this escapes any kind of special chars, but not a slash "/"
-    result = Kwave::Parser::escape(result);
-
-    // special handling for slashes "/" -> replace with unicode FRACTION SLASH
-    result.replace(QChar(0x002F), QChar(0x2044));
-
-    return result;
-}
-
 //***************************************************************************
 QString Kwave::SaveBlocksPlugin::createFileName(const QString &base,
     const QString &ext, const QString &pattern,
@@ -608,7 +582,7 @@ QString Kwave::SaveBlocksPlugin::createFileName(const QString &base,
 			    value = value + _(" ");
 		    }
 		}
-		value = escapeForFileName(value);
+		value = Kwave::Parser::escapeForFileName(value);
 	    }
 	}
 
@@ -626,7 +600,7 @@ QString Kwave::SaveBlocksPlugin::createFileName(const QString &base,
 	if ((idx >= 0) && (idx < m_block_info.count()))
 	    title = m_block_info[idx].m_title;
 	if (title.length()) {
-	    title = escapeForFileName(title);
+	    title = Kwave::Parser::escapeForFileName(title);
 	    p.replace(rx_title, title);
 	}
     }
